import urllib.parse
import logging
import wsgi_microservice_middleware


CORS_ALLOWED = wsgi_microservice_middleware.env.list("CORS_ALLOWED")


logger = logging.getLogger(__name__)


class CORSMiddleware(object):
    """
    Adds CORS headers to the responses of request that originate from domains inthe
    CORS_ALLOWED environment variable.  CORS_ALLOWED should reside in the environment as a
    comma-separated string of domain names.
    """
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):

        def custom_start_response(status, headers, exc_info=None):
            # append whatever headers you need here
            origin = urllib.parse.urlparse(environ.get('HTTP_ORIGIN'))
            if origin.netloc in CORS_ALLOWED:
                logger.info("Cross-origin request approved for request origin {0}".format(origin))
                headers.append(('Access-Control-Allow-Origin', '*'))
                headers.append(
                    ('Access-Control-Allow-Headers', '*')
                )
                headers.append(('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE, HEAD, OPTIONS'))
            return start_response(status, headers, exc_info)

        return self.app(environ, custom_start_response)


import environs

env = environs.Env()
env.read_env()

from wsgi_microservice_middleware.cors import CORSMiddleware
from wsgi_microservice_middleware.request_id import (
    RequestIdFilter, 
    RequestIdMiddleware, 
    current_request_id
)

__all__ = [
    'CORSMiddleware',
    'RequestIdFilter', 
    'RequestIdMiddleware', 
    'current_request_id'
]




"""
Middleware and logging filter to add request ids to logs and forward request Ids in downstream requests
"""
import logging
import traceback
import wsgi_microservice_middleware


logger = logging.getLogger(__name__)


REQUEST_ID_HEADER_NAME = wsgi_microservice_middleware.env.str("REQUEST_ID_HEADER", "X-Request-Id")


def make_wsgi_header_key(header: str):
    wsgi_header = "HTTP_" + REQUEST_ID_HEADER_NAME.replace("-","_").upper()
    return wsgi_header

class RequestIdMiddleware(object):
    """
    This middleware add access log-style record with a request id and includes
    the request Id in int he response headers
    """
    def __init__(self, app, header_name: str = None):
        self.header_name = header_name
        if not self.header_name:
            self.header_name = REQUEST_ID_HEADER_NAME
        self.wsgi_header_key = make_wsgi_header_key(self.header_name)
        self.app = app

    def __call__(self, environ, start_response):

        def custom_start_response(status, headers, exc_info=None):
            # append whatever headers you need here
            FACTS = [
                environ.get("HTTP_HOST", ""),
                environ.get("REQUEST_METHOD", ""),
                environ.get("RAW_URI", ""),
                environ.get("SERVER_PROTOCOL", ""),
                status
            ]
            message = " | ".join(FACTS)
            request_id = environ.get(self.wsgi_header_key, 'No Request Id')
            adpater = logging.LoggerAdapter(logger, extra={"request_id": request_id})
            adpater.info(message)


            headers.append((self.header_name, request_id,))

            return start_response(status, headers, exc_info)

        return self.app(environ, custom_start_response)



def current_request_id():
    """
    Retrives the current request id from the wsgi `environ` buried in the call stack
    """
    _req = None
    wsgi_header = "HTTP_" + REQUEST_ID_HEADER_NAME.replace("-","_").upper()
    try:
        for frame in traceback.walk_stack(None):
            if getattr(frame[0], 'f_globals', None) and getattr(frame[0], 'f_locals', None):
                if frame[0].f_globals.get('__name__', None) == __name__ and 'environ' in frame[0].f_locals:
                    environ = frame[0].f_locals['environ']
                    _req = environ.get(wsgi_header, None)
                    break
    except Exception:
        pass
    return _req


class RequestIdFilter(logging.Filter):
    """
    Logger filter to add a `{request_id}` logger variable tot he logging context
    """
    def __init__(self, header_name=REQUEST_ID_HEADER_NAME, *args, **kwargs):
        self.header_name = header_name
        self.wsgi_header_key = "HTTP_" + self.header_name.replace("-","_").upper()
        super().__init__(*args, **kwargs)

    def filter(self, record):
        record.request_id = self.get_current_request_id()
        return True

    def get_current_request_id(self):
        _req = current_request_id()
        if _req:
            request_id = _req
        else: 
            request_id = "No Request Id"
        return request_id





# coding: utf-8

"""
WSGI Microservice Middleware

Simple middlewares to help move microservices into production
"""


from setuptools import setup, find_packages  # noqa: H301

NAME = "wsgi_microservice_middleware"
VERSION = "0.1.0"

REQUIRES = [
    "urllib3 >= 1.15",
    "environs >= 4.2.0",
]

with open("README.md", "r") as fh:
    long_description = fh.read()
    

setup(
    name=NAME,
    version=VERSION,
    description="WSGI Microservice Middleware",
    author_email="inquiries@presalytics.io",
    url="https://github.com/presalytics/WSGI-Microservice-Middleware",
    keywords=["WSGI", "middleware", "microservice"],
    install_requires=REQUIRES,
    packages=find_packages(),
    license="MIT",
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description=long_description,
    python_requires=">=3.5",
    project_urls = {
        "Issues": "https://github.com/presalytics/WSGI-Microservice-Middleware/issues"
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        "Natural Language :: English", 
        "Development Status :: 4 - Beta",
    ]
)
